package webdb.addressbook.service.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.seasar.flex2.core.format.amf3.type.ByteArray;
import org.seasar.flex2.rpc.remoting.service.annotation.RemotingService;
import org.seasar.flex2.util.data.storage.StorageType;
import org.seasar.flex2.util.data.transfer.annotation.Export;
import org.seasar.flex2.util.data.transfer.annotation.Import;
import org.seasar.framework.beans.annotation.ParameterName;
import org.seasar.framework.util.FileOutputStreamUtil;

import webdb.addressbook.config.ServiceConfig;
import webdb.addressbook.dto.Card;
import webdb.addressbook.service.AddressBookService;

/**
 * <h4>AddressBookServiceImpl</h4>
 * AddressBookService̎NXłB
 */
@RemotingService
public class AddressBookServiceImpl implements AddressBookService {
	/** Serviceݒ */
	private ServiceConfig serviceConfig;
	/** J[hꗗ */
	private List<Card> cardList;
	
	/**
	 * @return the cardList
	 * ̃f[^́ASessionf[^Exportꂽf[^Zbg܂B
	 */
    @Export(storage = StorageType.SESSION)
	public List<Card> getCardList() {
		return cardList;
	}

	/**
	 * @param cardList the cardList to set
	 *@̃f[^̓ZbVf[^Import܂B
	 */
    @Import(storage = StorageType.SESSION)
	public void setCardList(@ParameterName("cardList") List<Card> cardList) {
		this.cardList = cardList;
	}

	/* (non-Javadoc)
	 * @see webdb.addressbook.service.AddressBookService#getCards()
	 */
	public Card[] getCards() {
		
		if(cardList == null){
			cardList = new ArrayList<Card>();
			
		}
		
		return cardList.toArray(new Card[]{});
	}
	
	/* (non-Javadoc)
	 * @see webdb.addressbook.service.AddressBookService#deleteCard(java.lang.String)
	 * @ParameterName cardId;
	 */
	public int deleteCard(@ParameterName("cardId") int cardId) {
		/*
		if(cardList.size()>=cardId){
			Card card =cardList.get(cardId-1);
			deletePortraitFileIfExists(card);
			cardList.remove(cardId-1);
			return 1;
		}
		*/
		for (int i=0;i<cardList.size();i++){
			Card card = cardList.get(i);
			if(card.getId()==cardId){
				deletePortraitFileIfExists(card);
				cardList.remove(i);
				return 1;
			}
		}
		return 0;
	}
	/* (non-Javadoc)
	 * @see webdb.addressbook.service.AddressBookService#saveCard(webdb.addressbook.dto.Card)
	 */
	public void saveCard(Card card) {
		if(card == null) return;
		card.setPortraitUrl(saveByteArrayAsImageFile(card));
		if(card.getId()<=0){
			card.setId(cardList.size()+1);
			cardList.add(card);
		}else{
			cardList.set(card.getId()-1, card);
		}
		
	}
	
	/**
	 * w肳ꂽCardIuWFNg̎G摜t@C폜܂B
	 */
	private void deletePortraitFileIfExists( Card card) {
		if(card == null ) return;
		if(card.getPortraitUrl()!= null){
			File f = new File(getPortraitFilepath(card));
			if(f.exists()){
				f.delete();
			}
		}
	}

	/**
	 * G摜t@Ĉ̖𐶐܂B
	 */
	private String generatePortraitFile() {
		return System.currentTimeMillis() + ".png";	
	}

	/**
	 * G摜t@C̃t@CpXԂ܂B
	 */
	private String getPortraitFilepath(Card card) {
		return serviceConfig.getRootPath()+resolvPortraitFile(card);
	}

	/**
	 * w肳ꂽCardIuWFNgɎGf[^
	 * t@CɕۑA摜URLԂ܂B
	 */
	private String saveByteArrayAsImageFile(Card card) {
	    String fileName = card.getPortraitUrl()==null?generatePortraitFile():resolvPortraitFile(card);
		ByteArray byteArray=card.getPortrait();
		if(byteArray == null) {
			if(card.getPortraitUrl()== null){
				return null;
			}
			return card.getPortraitUrl();
		}
		
        byte[] buffer = byteArray.getBufferBytes();
        
        File f = new File(serviceConfig.getRootPath()+fileName);
        
        FileOutputStream fileOutputSteam = FileOutputStreamUtil.create(f);
        try {
            fileOutputSteam.write(buffer);
            fileOutputSteam.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (fileOutputSteam != null) {
                try {
                    fileOutputSteam.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }   
        String url = serviceConfig.getRootUri()+fileName;
		return url;
	}

	/**
	 * G̃t@C擾܂B
	 * @param card
	 * @return@G̃t@C
	 */
	private String resolvPortraitFile(Card card) {
		String url= card.getPortraitUrl();
		if(url.indexOf("/")>-1){
			return url.substring(url.lastIndexOf("/")+1);
		}
		return url;
	}

	/**
	 * @return the serviceConfig
	 */
	public ServiceConfig getServiceConfig() {
		return serviceConfig;
	}

	/**
	 * @param serviceConfig the serviceConfig to set
	 */
	public void setServiceConfig(ServiceConfig serviceConfig) {
		this.serviceConfig = serviceConfig;
	}

	public int deleteCard(Card card) {
		if(card==null) return 0;
		return deleteCard(card.getId());
	}

}
